@@ -1,7 +1,10 @@
+import asyncio
+import logging
 import math
 import random
 
-from typing import TYPE_CHECKING
+from collections.abc import AsyncGenerator
+from typing import TYPE_CHECKING, Literal
 
 from pydantic import BaseModel, ConfigDict, Field
 
@@ -19,14 +22,32 @@ from .constants import (
 from .dataset import Dataset
 from .exceptions import DataSetGeneratorError
 from .llm import LLMClient
-from .prompts import CONVERSATION_GENERATION_PROMPT
-from .schemas import ChatTranscript
+from .metrics import trace
+from .prompts import (
+    AGENT_COT_HYBRID_PROMPT,
+    AGENT_COT_MULTI_TURN_PROMPT,
+    AGENT_COT_TOOLS_PROMPT,
+    CONVERSATION_GENERATION_PROMPT,
+    FREETEXT_COT_MATHEMATICAL_PROMPT,
+    FREETEXT_COT_PROMPT,
+    HYBRID_COT_MATHEMATICAL_PROMPT,
+    HYBRID_COT_PROMPT,
+    STRUCTURED_COT_MATHEMATICAL_PROMPT,
+    STRUCTURED_COT_PROMPT,
+    XLAM_MULTI_TURN_PROMPT,
+    AgentPromptBuilder,
+)
+from .schemas import ToolRegistry, get_conversation_schema
+from .tools.loader import get_available_tools, load_tools_from_dict, load_tools_from_file
 from .topic_model import TopicModel
+from .utils import ensure_not_running_loop
 
 # Handle circular import for type hints
 if TYPE_CHECKING:
     from .topic_model import TopicModel
 
+logger = logging.getLogger(__name__)
+
 
 class DataSetGeneratorConfig(BaseModel):
     """Configuration for the data engine."""
@@ -61,6 +82,9 @@ class DataSetGeneratorConfig(BaseModel):
         le=10,
         description="Maximum number of retries for failed requests",
     )
+    max_tokens: int = Field(
+        default=2000, ge=1, description="Maximum tokens to generate in a single call to the llm"
+    )
     default_batch_size: int = Field(
         default=ENGINE_DEFAULT_BATCH_SIZE,
         ge=1,
@@ -78,6 +102,42 @@ class DataSetGeneratorConfig(BaseModel):
     )
     sys_msg: bool = Field(default=True, description="Whether to include system message in dataset")
 
+    # Chain of Thought parameters
+    conversation_type: Literal[
+        "basic",
+        "structured",
+        "tool_calling",
+        "cot_freetext",
+        "cot_structured",
+        "cot_hybrid",
+        "agent_cot_tools",
+        "agent_cot_hybrid",
+        "agent_cot_multi_turn",
+        "xlam_multi_turn",
+    ] = Field(default="basic", description="Type of conversation to generate")
+    reasoning_style: Literal["mathematical", "logical", "general"] = Field(
+        default="general", description="Style of reasoning for CoT generation"
+    )
+
+    # Agent-specific parameters
+    available_tools: list[str] = Field(
+        default_factory=list,
+        description="List of tool names available to the agent (empty means all tools)",
+    )
+    custom_tools: list[dict] = Field(
+        default_factory=list, description="Custom tool definitions as dictionaries"
+    )
+    max_tools_per_query: int = Field(
+        default=3, ge=1, le=10, description="Maximum number of tools an agent can use per query"
+    )
+    tool_registry_path: str | None = Field(
+        default=None, description="Path to custom tool definitions file (JSON/YAML)"
+    )
+    domain_context: str | None = Field(
+        default=None,
+        description="Domain/scenario context for XLAM multi-turn (becomes 'system' field)",
+    )
+
 
 class DataSetGenerator:
     def __init__(self, **kwargs):
@@ -99,6 +159,7 @@ class DataSetGenerator:
             provider=self.provider,
             model_name=self.model_name,
         )
+        trace("generator_created", {"provider": self.provider})
 
         # Store dataset system prompt for dataset inclusion (with fallback)
         self.dataset_system_prompt = (
@@ -107,6 +168,45 @@ class DataSetGenerator:
         # Store generation prompt for content generation
         self.generation_prompt = self.config.generation_system_prompt
 
+        # Initialize tool registry for agent conversation types
+        self.tool_registry = None
+        if self.config.conversation_type in {
+            "agent_cot_tools",
+            "agent_cot_hybrid",
+            "agent_cot_multi_turn",
+            "xlam_multi_turn",
+        }:
+            self._initialize_tool_registry()
+
+    def _initialize_tool_registry(self):
+        """Initialize tool registry from configuration."""
+        try:
+            custom_registry = None
+
+            # Load tools from file if specified
+            if self.config.tool_registry_path:
+                custom_registry = load_tools_from_file(self.config.tool_registry_path)
+
+            # Load custom tools from dict and merge with file-based tools if both exist
+            if self.config.custom_tools:
+                dict_registry = load_tools_from_dict(self.config.custom_tools)
+                if custom_registry:
+                    # Merge both registries - tools from dict take precedence
+                    # Create a new registry with combined tools
+                    combined_tools = custom_registry.tools + dict_registry.tools
+                    custom_registry = ToolRegistry(tools=combined_tools)
+                else:
+                    custom_registry = dict_registry
+
+            # Get available tools based on configuration
+            self.tool_registry = get_available_tools(
+                available_tool_names=self.config.available_tools or None,
+                custom_registry=custom_registry,
+            )
+
+        except Exception as e:
+            raise DataSetGeneratorError(f"Failed to initialize tool registry: {str(e)}") from e
+
     def _validate_create_data_params(
         self,
         num_steps: int,
@@ -115,13 +215,15 @@ class DataSetGenerator:
     ) -> None:
         """Validate parameters for data creation."""
         if num_steps is None or num_steps <= 0:
-            raise DataSetGeneratorError("positive")
+            raise DataSetGeneratorError("num_steps must be a positive integer")
 
         if batch_size <= 0:
-            raise DataSetGeneratorError("positive")
+            raise DataSetGeneratorError("batch_size must be a positive integer")
 
         if topic_model and len(topic_model.get_all_paths()) == 0:
-            raise DataSetGeneratorError("")
+            raise DataSetGeneratorError(
+                "Topic model has no paths. Ensure the topic tree was built successfully."
+            )
 
     def _prepare_topic_paths(
         self,
@@ -186,31 +288,37 @@ class DataSetGenerator:
             prompts.append(sample_prompt)
         return prompts
 
-    def _generate_structured_samples(
+    async def _generate_structured_samples_async(
         self,
         prompts: list[str],
         include_sys_msg: bool,
     ) -> tuple[list, list]:
-        """Generate structured samples using Outlines."""
+        """Generate structured samples using Outlines with appropriate schema."""
         samples = []
         failed_responses = []
 
-        for prompt in prompts:
+        schema_class = get_conversation_schema(
+            self.config.conversation_type, self.config.reasoning_style
+        )
+
+        async def _generate(prompt: str) -> tuple[bool, dict | Exception]:
             try:
-                # Generate structured conversation using ChatTranscript schema
-                conversation = self.llm_client.generate(
+                # Use higher of max_tokens, 4000 tokens for multi-turn conversations
+                max_tokens = self.config.max_tokens
+                if self.config.conversation_type == "xlam_multi_turn":
+                    max_tokens = max(max_tokens, 4000)
+
+                conversation = await self.llm_client.generate_async(
                     prompt=prompt,
-                    schema=ChatTranscript,
+                    schema=schema_class,
                     max_retries=self.config.max_retries,
-                    max_tokens=2000,
+                    max_tokens=max_tokens,
                     temperature=self.config.temperature,
                 )
 
-                # Convert Pydantic model to dict
                 sample = conversation.model_dump()
 
-                # Add system message at the start if sys_msg is True
-                if include_sys_msg:
+                if include_sys_msg and "messages" in sample:
                     sample["messages"].insert(
                         0,
                         {
@@ -219,12 +327,47 @@ class DataSetGenerator:
                         },
                     )
 
-                samples.append(sample)
+                if (
+                    self.config.conversation_type
+                    in {
+                        "agent_cot_tools",
+                        "agent_cot_hybrid",
+                        "agent_cot_multi_turn",
+                        "xlam_multi_turn",
+                    }
+                    and self.tool_registry is not None
+                ):
+                    sample["available_tools"] = [
+                        tool.model_dump() for tool in self.tool_registry.tools
+                    ]
 
-            except Exception as e:
-                error_msg = f"Generation failed: {str(e)}"
+                # Add domain context for XLAM multi-turn
+                if self.config.conversation_type == "xlam_multi_turn":
+                    if self.config.domain_context:
+                        sample["domain_context"] = self.config.domain_context
+                    elif "scenario_description" not in sample:
+                        sample["scenario_description"] = "AI assistant conversation"
+
+            except Exception as e:  # noqa: BLE001
+                if self.config.conversation_type == "xlam_multi_turn":
+                    logger.error("XLAM generation error: %s", e, exc_info=True)
+                return False, e
+            else:
+                return True, sample
+
+        tasks = [asyncio.create_task(_generate(prompt)) for prompt in prompts]
+        results = await asyncio.gather(*tasks)
+
+        for success, payload in results:
+            if success:
+                samples.append(payload)  # type: ignore[arg-type]
+            else:
+                error = payload
+                error_msg = f"Generation failed: {error}"
                 failed_responses.append(error_msg)
-                failure_type = self.analyze_failure(str(e), error=e)
+                failure_type = self.analyze_failure(
+                    str(error), error=error if isinstance(error, Exception) else None
+                )
                 self.failure_analysis[failure_type].append(error_msg)
 
         return samples, failed_responses
@@ -256,11 +399,11 @@ class DataSetGenerator:
         }
 
         # Add example failures for each category
-        for _category, failures in self.failure_analysis.items():
+        for category, failures in self.failure_analysis.items():
             if failures:
                 # Get up to 3 examples for each category
                 examples = failures[:3]
-                summary["failure_examples"] = [
+                summary["failure_examples"][category] = [
                     (
                         str(ex)[:200] + "..."
                         if len(str(ex)) > 200  # noqa: PLR2004
@@ -279,47 +422,18 @@ class DataSetGenerator:
         model_name: str | None = None,
         sys_msg: bool | None = None,
     ):
-        # Set default value for num_steps if None
-        if num_steps is None:
-            num_steps = 1
-
-        # Validate inputs
-        self._validate_create_data_params(num_steps, batch_size, topic_model)
-
-        # Use instance model_name as fallback if none provided
-        if model_name:
-            self.model_name = model_name.strip()
-
-        if not self.model_name:
-            raise DataSetGeneratorError("")
-
-        # Use provided sys_msg or fall back to config.sys_msg
-        include_sys_msg = sys_msg if sys_msg is not None else self.config.sys_msg
-
-        # Prepare topic paths and adjust num_steps if necessary
-        topic_paths, num_steps = self._prepare_topic_paths(num_steps, batch_size, topic_model)
-
-        total_samples = num_steps * batch_size
-        data_creation_prompt = CONVERSATION_GENERATION_PROMPT
-
-        # Use generator pattern for progress events (no TUI dependencies)
-        generator = self._run_generation_loop(
-            num_steps=num_steps,
-            batch_size=batch_size,
-            total_samples=total_samples,
-            topic_paths=topic_paths or [],
-            data_creation_prompt=data_creation_prompt,
-            num_example_demonstrations=num_example_demonstrations,
-            include_sys_msg=include_sys_msg,
+        ensure_not_running_loop("DataSetGenerator.create_data")
+        return asyncio.run(
+            self.create_data_async(
+                num_steps=num_steps,
+                num_example_demonstrations=num_example_demonstrations,
+                batch_size=batch_size,
+                topic_model=topic_model,
+                model_name=model_name,
+                sys_msg=sys_msg,
+            )
         )
 
-        # Consume the generator and return the final dataset
-        final_result = None
-        for event in generator:
-            final_result = event
-
-        return final_result
-
     def create_data_with_events(
         self,
         num_steps: int | None = None,
@@ -329,45 +443,66 @@ class DataSetGenerator:
         model_name: str | None = None,
         sys_msg: bool | None = None,
     ):
-        """Create dataset yielding progress events (for TUI integration)."""
-        # Set default value for num_steps if None
+        ensure_not_running_loop("DataSetGenerator.create_data_with_events")
+
+        async def _async_generator() -> AsyncGenerator[dict | Dataset, None]:
+            async for event in self.create_data_with_events_async(
+                num_steps=num_steps,
+                num_example_demonstrations=num_example_demonstrations,
+                batch_size=batch_size,
+                topic_model=topic_model,
+                model_name=model_name,
+                sys_msg=sys_msg,
+            ):
+                yield event
+
+        agen = _async_generator()
+
+        def _sync_generator():
+            loop = asyncio.new_event_loop()
+            try:
+                while True:
+                    try:
+                        event = loop.run_until_complete(agen.__anext__())
+                    except StopAsyncIteration:
+                        break
+                    else:
+                        yield event
+            finally:
+                loop.run_until_complete(agen.aclose())
+                loop.close()
+
+        return _sync_generator()
+
+    async def create_data_async(
+        self,
+        num_steps: int | None = None,
+        num_example_demonstrations: int = 3,
+        batch_size: int = 10,
+        topic_model: TopicModel | None = None,
+        model_name: str | None = None,
+        sys_msg: bool | None = None,
+    ) -> Dataset:
         if num_steps is None:
             num_steps = 1
 
-        # Validate inputs
         self._validate_create_data_params(num_steps, batch_size, topic_model)
 
-        # Use instance model_name as fallback if none provided
         if model_name:
             self.model_name = model_name.strip()
 
         if not self.model_name:
             raise DataSetGeneratorError("")
 
-        # Use provided sys_msg or fall back to config.sys_msg
         include_sys_msg = sys_msg if sys_msg is not None else self.config.sys_msg
 
-        # Prepare topic paths and adjust num_steps if necessary
-        topic_paths, num_steps = self._prepare_topic_paths(num_steps, batch_size, topic_model)
-
-        # Use instance model_name as fallback if none provided
-        if model_name:
-            self.model_name = model_name.strip()
-
-        if not self.model_name:
-            raise DataSetGeneratorError("")
-
-        # Use provided sys_msg or fall back to config.sys_msg
-        include_sys_msg = sys_msg if sys_msg is not None else self.config.sys_msg
-
-        # Prepare topic paths and adjust num_steps if necessary
         topic_paths, num_steps = self._prepare_topic_paths(num_steps, batch_size, topic_model)
 
         total_samples = num_steps * batch_size
-        data_creation_prompt = CONVERSATION_GENERATION_PROMPT
+        data_creation_prompt = self._get_cot_prompt_template()
 
-        # Yield from the generation loop
-        yield from self._run_generation_loop(
+        final_result: Dataset | dict | None = None
+        async for event in self._run_generation_loop_async(
             num_steps=num_steps,
             batch_size=batch_size,
             total_samples=total_samples,
@@ -375,9 +510,62 @@ class DataSetGenerator:
             data_creation_prompt=data_creation_prompt,
             num_example_demonstrations=num_example_demonstrations,
             include_sys_msg=include_sys_msg,
-        )
+        ):
+            final_result = event
 
-    def _run_generation_loop(  # noqa: PLR0912
+        if isinstance(final_result, Dataset):
+            trace(
+                "dataset_created",
+                {
+                    "samples_count": len(final_result.samples),
+                    "failed_samples": len(self.failed_samples),
+                    "success": len(final_result.samples) > 0,
+                },
+            )
+            return final_result
+
+        msg = "Dataset generation failed"
+        raise DataSetGeneratorError(msg)
+
+    async def create_data_with_events_async(
+        self,
+        num_steps: int | None = None,
+        num_example_demonstrations: int = 3,
+        batch_size: int = 10,
+        topic_model: TopicModel | None = None,
+        model_name: str | None = None,
+        sys_msg: bool | None = None,
+    ) -> AsyncGenerator[dict | Dataset, None]:
+        if num_steps is None:
+            num_steps = 1
+
+        self._validate_create_data_params(num_steps, batch_size, topic_model)
+
+        if model_name:
+            self.model_name = model_name.strip()
+
+        if not self.model_name:
+            raise DataSetGeneratorError("")
+
+        include_sys_msg = sys_msg if sys_msg is not None else self.config.sys_msg
+
+        topic_paths, num_steps = self._prepare_topic_paths(num_steps, batch_size, topic_model)
+
+        total_samples = num_steps * batch_size
+        data_creation_prompt = self._get_cot_prompt_template()
+
+        async for event in self._run_generation_loop_async(
+            num_steps=num_steps,
+            batch_size=batch_size,
+            total_samples=total_samples,
+            topic_paths=topic_paths or [],
+            data_creation_prompt=data_creation_prompt,
+            num_example_demonstrations=num_example_demonstrations,
+            include_sys_msg=include_sys_msg,
+        ):
+            yield event
+
+    async def _run_generation_loop_async(  # noqa: PLR0912
         self,
         num_steps: int,
         batch_size: int,
@@ -386,10 +574,9 @@ class DataSetGenerator:
         data_creation_prompt: str,
         num_example_demonstrations: int,
         include_sys_msg: bool,
-    ):
+    ) -> AsyncGenerator[dict | Dataset, None]:
         """Run the main generation loop yielding progress events."""
         try:
-            # Yield start event
             yield {
                 "event": "generation_start",
                 "model_name": self.model_name,
@@ -410,15 +597,25 @@ class DataSetGenerator:
                     num_example_demonstrations,
                 )
 
-                success, samples_generated = self._process_batch_with_retries(
+                failed_before = len(self.failed_samples)
+
+                success, samples_generated = await self._process_batch_with_retries_async(
                     prompts, include_sys_msg
                 )
 
+                failed_in_batch = len(self.failed_samples) - failed_before
+                failure_reasons = []
+                if failed_in_batch > 0 and self.failed_samples:
+                    recent_failures = self.failed_samples[-failed_in_batch:]
+                    failure_reasons = recent_failures[:3]
+
                 yield {
                     "event": "step_complete",
                     "step": step + 1,
                     "samples_generated": samples_generated,
                     "success": success,
+                    "failed_in_step": failed_in_batch,
+                    "failure_reasons": failure_reasons,
                 }
 
                 if not success:
@@ -439,16 +636,15 @@ class DataSetGenerator:
             self.print_failure_summary()
             self.save_dataset(INTERRUPTED_DATASET_FILENAME)
 
-        except Exception as e:
+        except Exception as e:  # noqa: BLE001
             yield {"event": "generation_error", "error": str(e)}
             self.print_failure_summary()
             self.save_dataset(ERROR_DATASET_FILENAME)
             raise DataSetGeneratorError("failed") from e
 
-        # Always return the dataset as the final result
         yield self.dataset
 
-    def _process_batch_with_retries(
+    async def _process_batch_with_retries_async(
         self,
         prompts: list[str],
         include_sys_msg: bool,
@@ -456,7 +652,7 @@ class DataSetGenerator:
         """Process a batch with retry logic."""
         for attempt in range(self.config.max_retries):
             try:
-                samples, failed_responses = self._generate_structured_samples(
+                samples, failed_responses = await self._generate_structured_samples_async(
                     prompts, include_sys_msg
                 )
 
@@ -465,14 +661,15 @@ class DataSetGenerator:
 
                 if samples:
                     failed_samples, failure_descriptions = self.dataset.add_samples(samples)
+
                     if failed_samples:
                         for sample, desc in zip(failed_samples, failure_descriptions, strict=True):
                             self.failed_samples.append(sample)
                             self.failure_analysis["invalid_schema"].append(desc)
 
                     successful_samples = len(samples) - len(failed_samples)
-
                     return True, successful_samples  # Success - exit retry loop
+
             except DataSetGeneratorError as e:
                 # Authentication and API errors are now wrapped in DataSetGeneratorError
                 error_str = str(e).lower()
@@ -487,9 +684,7 @@ class DataSetGenerator:
                     self.failure_analysis["api_errors"].append(error_msg)
 
                 self.failed_samples.append(error_msg)
-
-                print(f"Error: {error_msg}")
-
+                logger.exception("API error: %s", error_msg)
                 return False, 0  # Don't retry authentication/API errors
             except Exception as e:
                 if attempt == self.config.max_retries - 1:
@@ -497,6 +692,9 @@ class DataSetGenerator:
                     failure_type = self.analyze_failure(str(e), error=e)
                     self.failure_analysis[failure_type].append(str(e))
                     return False, 0
+            else:
+                # If no exception and no samples, return False, 0
+                return False, 0
 
         return False, 0
 
@@ -527,7 +725,18 @@ class DataSetGenerator:
         prompt = prompt.replace(
             "{{{{examples}}}}", self.build_examples_text(num_example_demonstrations)
         )
-        return prompt.replace("{{{{subtopics}}}}", self.build_subtopics_text(subtopics_list))
+        prompt = prompt.replace("{{{{subtopics}}}}", self.build_subtopics_text(subtopics_list))
+
+        # XLAM multi-turn specific replacements
+        if self.config.conversation_type == "xlam_multi_turn":
+            tools_text = self.build_tools_text()
+            domain_context = (
+                self.config.domain_context or "AI assistant with access to various tools"
+            )
+            prompt = prompt.replace("{{{{tools}}}}", tools_text)
+            prompt = prompt.replace("{{{{domain_context}}}}", domain_context)
+
+        return prompt
 
     def build_system_prompt(self):
         """Return the original system prompt for dataset inclusion."""
@@ -547,11 +756,71 @@ class DataSetGenerator:
         examples_text += "\n".join(f"Example {i + 1}: \n\n{ex}\n" for i, ex in enumerate(examples))
         return f"\nHere are output examples:\n<examples>\n{examples_text}\n</examples>\n"
 
+    def build_tools_text(self) -> str:
+        """Build formatted tools text for XLAM multi-turn prompts."""
+        if not self.tool_registry:
+            return "No tools available"
+
+        tools_text = []
+        for tool in self.tool_registry.tools:
+            params_text = []
+            for param in tool.parameters:
+                req = " (required)" if param.required else " (optional)"
+                params_text.append(f"  - {param.name} ({param.type}){req}: {param.description}")
+
+            tool_text = f"• {tool.name}: {tool.description}\n  Parameters:\n" + "\n".join(
+                params_text
+            )
+            tools_text.append(tool_text)
+
+        return "\n\n".join(tools_text)
+
     def build_subtopics_text(self, subtopic_list: list[str] | None):
         if subtopic_list is None:
             return ""
         return f"\nLastly, the topic of the training data should be related to the following subtopics: {' -> '.join(subtopic_list)}"
 
+    def _get_cot_prompt_template(self) -> str:  # noqa: PLR0911
+        """Get the appropriate CoT prompt template based on conversation type and reasoning style."""
+        # Use mathematical prompts when reasoning_style is mathematical
+        if self.config.reasoning_style == "mathematical":
+            mathematical_prompts = {
+                "cot_freetext": FREETEXT_COT_MATHEMATICAL_PROMPT,
+                "cot_structured": STRUCTURED_COT_MATHEMATICAL_PROMPT,
+                "cot_hybrid": HYBRID_COT_MATHEMATICAL_PROMPT,
+            }
+            if self.config.conversation_type in mathematical_prompts:
+                return mathematical_prompts[self.config.conversation_type]
+
+        # Agent tool-calling prompts use structured generation
+        if self.config.conversation_type == "agent_cot_tools":
+            if self.tool_registry is not None:
+                return AgentPromptBuilder.build_tool_context_prompt(self.tool_registry)
+            return AGENT_COT_TOOLS_PROMPT
+
+        if self.config.conversation_type == "agent_cot_hybrid":
+            if self.tool_registry is not None:
+                return AgentPromptBuilder.build_tool_context_prompt(self.tool_registry)
+            return AGENT_COT_HYBRID_PROMPT
+
+        if self.config.conversation_type == "agent_cot_multi_turn":
+            if self.tool_registry is not None:
+                return AgentPromptBuilder.build_multi_turn_context_prompt(self.tool_registry)
+            return AGENT_COT_MULTI_TURN_PROMPT
+
+        if self.config.conversation_type == "xlam_multi_turn":
+            # For XLAM multi-turn, we use a custom prompt with tools and domain context
+
+            return XLAM_MULTI_TURN_PROMPT
+
+        # Default CoT prompts for non-mathematical reasoning
+        cot_prompts = {
+            "cot_freetext": FREETEXT_COT_PROMPT,
+            "cot_structured": STRUCTURED_COT_PROMPT,
+            "cot_hybrid": HYBRID_COT_PROMPT,
+        }
+        return cot_prompts.get(self.config.conversation_type, CONVERSATION_GENERATION_PROMPT)
+
     def save_dataset(self, save_path: str):
         """Save the dataset to a file."""
         self.dataset.save(save_path)
