@@ -1,3 +1,4 @@
+import asyncio
 import json
 import time
 import warnings
@@ -16,7 +17,8 @@ from .constants import (
 )
 from .exceptions import TreeError
 from .llm import LLMClient
-from .prompts import TOPIC_EXPANSION_PROMPT
+from .metrics import trace
+from .prompts import TreePromptBuilder
 from .schemas import TopicList
 from .topic_model import TopicModel
 
@@ -145,12 +147,16 @@ class Tree(TopicModel):
             model_name=self.model_name,
         )
 
+        trace(
+            "tree_created", {"provider": self.provider, "degree": self.degree, "depth": self.depth}
+        )
+
         # Derived attributes
         self.system_prompt = self.config.topic_system_prompt
         self.tree_paths: list[list[str]] = []
         self.failed_generations: list[dict[str, Any]] = []
 
-    def build(self, model_name: str | None = None):
+    async def build_async(self, model_name: str | None = None):
         """Build the complete topic tree.
 
         Args:
@@ -176,17 +182,27 @@ class Tree(TopicModel):
                 raise RuntimeError(error_msg)
 
         try:
-            yield from self._build_subtree_generator(
+            async for event in self._build_subtree_generator(
                 [self.config.topic_prompt],
                 self.config.topic_system_prompt,
                 self.config.depth,
                 self.config.degree,
                 1,
-            )
+            ):
+                yield event
 
             # Check if build was completely unsuccessful (no paths generated)
             _raise_if_build_failed()
 
+            trace(
+                "tree_built",
+                {
+                    "total_paths": len(self.tree_paths),
+                    "failed_generations": len(self.failed_generations),
+                    "success": len(self.tree_paths) > 0,
+                },
+            )
+
             yield {
                 "event": "build_complete",
                 "total_paths": len(self.tree_paths),
@@ -203,19 +219,24 @@ class Tree(TopicModel):
         """Returns all the paths in the topic model."""
         return self.tree_paths
 
-    def get_subtopics(
+    async def get_subtopics(
         self, system_prompt: str, node_path: list[str], num_subtopics: int
     ) -> list[str]:
         """Generate subtopics using structured generation."""
 
-        prompt = TOPIC_EXPANSION_PROMPT
-        prompt = prompt.replace("{{{{system_prompt}}}}", system_prompt if system_prompt else "")
-        prompt = prompt.replace("{{{{subtopics_list}}}}", " -> ".join(node_path))
-        prompt = prompt.replace("{{{{num_subtopics}}}}", str(num_subtopics))
+        # Determine domain based on system prompt or path content
+        domain = self._detect_domain(system_prompt, node_path)
+
+        prompt = TreePromptBuilder.build_expansion_prompt(
+            topic_path=node_path,
+            num_subtopics=num_subtopics,
+            system_prompt=system_prompt if system_prompt else "",
+            domain=domain,
+        )
 
         try:
             # Generate structured subtopics using TopicList schema
-            topic_response = self.llm_client.generate(
+            topic_response = await self.llm_client.generate_async(
                 prompt=prompt,
                 schema=TopicList,
                 max_retries=MAX_RETRY_ATTEMPTS,
@@ -247,7 +268,28 @@ class Tree(TopicModel):
             # Generate default subtopics
             return [f"subtopic_{i + 1}_for_{node_path[-1]}" for i in range(num_subtopics)]
 
-    def _build_subtree_generator(
+    def _detect_domain(self, system_prompt: str, node_path: list[str]) -> str:
+        """Detect the appropriate domain for prompt examples based on context."""
+        combined_text = f"{system_prompt} {' '.join(node_path)}".lower()
+
+        if any(
+            word in combined_text
+            for word in ["math", "calculus", "algebra", "geometry", "equation"]
+        ):
+            return "educational"
+        if any(
+            word in combined_text
+            for word in ["programming", "code", "software", "python", "algorithm"]
+        ):
+            return "technical"
+        if any(
+            word in combined_text
+            for word in ["chat", "conversation", "talk", "friendly", "assistant"]
+        ):
+            return "conversational"
+        return "general"
+
+    async def _build_subtree_generator(
         self,
         node_path: list[str],
         system_prompt: str,
@@ -274,25 +316,42 @@ class Tree(TopicModel):
             yield {"event": "leaf_reached", "path": node_path}
             return
 
-        subtopics = self.get_subtopics(system_prompt, node_path, n_child)
+        subtopics = await self.get_subtopics(system_prompt, node_path, n_child)
 
-        yield {
+        event = {
             "event": "subtopics_generated",
             "parent_path": node_path,
             "count": len(subtopics),
             "success": bool(subtopics),
         }
 
+        # Include error information if generation failed
+        if not event["success"] and self.failed_generations:
+            # Get the most recent failure
+            recent_failure = self.failed_generations[-1]
+            event["error"] = recent_failure.get("error", "Unknown error")
+
+        yield event
+
         if not subtopics:
             self.tree_paths.append(node_path)
             yield {"event": "leaf_reached", "path": node_path}
             return
 
-        for subtopic in subtopics:
-            child_path = node_path + [subtopic]
-            yield from self._build_subtree_generator(
+        async def _collect_child_events(child_subtopic: str) -> list[dict[str, Any]]:
+            child_path = node_path + [child_subtopic]
+            events: list[dict[str, Any]] = []
+            async for child_event in self._build_subtree_generator(
                 child_path, system_prompt, total_depth, n_child, current_depth + 1
-            )
+            ):
+                events.append(child_event)
+            return events
+
+        tasks = [asyncio.create_task(_collect_child_events(subtopic)) for subtopic in subtopics]
+
+        for child_events in await asyncio.gather(*tasks):
+            for child_event in child_events:
+                yield child_event
 
     def save(self, save_path: str) -> None:
         """Save the topic tree to a file."""
