@@ -32,6 +32,8 @@ def sample_config_dict():
             "model": "model",
             "temperature": 0.9,
             "max_retries": 2,
+            "conversation_type": "basic",
+            "reasoning_style": "general",
         },
         "dataset": {
             "creation": {
@@ -42,6 +44,7 @@ def sample_config_dict():
                 "sys_msg": True,
             },
             "save_as": "test_dataset.jsonl",
+            "formatters": [],
         },
     }
 
@@ -68,6 +71,8 @@ def sample_config_dict_no_sys_msg():
             "model": "model",
             "temperature": 0.9,
             "max_retries": 2,
+            "conversation_type": "basic",
+            "reasoning_style": "general",
         },
         "dataset": {
             "creation": {
@@ -77,6 +82,7 @@ def sample_config_dict_no_sys_msg():
                 "model": "model",
             },
             "save_as": "test_dataset.jsonl",
+            "formatters": [],
         },
     }
 
@@ -114,9 +120,18 @@ def test_load_from_yaml(sample_yaml_file, sample_config_dict):
     config = DeepFabricConfig.from_yaml(sample_yaml_file)
 
     assert config.dataset_system_prompt == sample_config_dict["dataset_system_prompt"]
-    assert config.topic_tree == sample_config_dict["topic_tree"]
-    assert config.data_engine == sample_config_dict["data_engine"]
-    assert config.dataset == sample_config_dict["dataset"]
+    assert config.topic_tree.model_dump(exclude_none=True) == sample_config_dict["topic_tree"]
+
+    # Add the new agent fields to the expected config for comparison
+    expected_data_engine = sample_config_dict["data_engine"].copy()
+    expected_data_engine["available_tools"] = []  # Default value
+    expected_data_engine["custom_tools"] = []  # Default value
+    expected_data_engine["max_tools_per_query"] = 3  # Default value
+    expected_data_engine["max_tokens"] = 2000
+
+    actual_data_engine = config.data_engine.model_dump(exclude_none=True)
+    assert actual_data_engine == expected_data_engine
+    assert config.dataset.model_dump(exclude_none=True) == sample_config_dict["dataset"]
 
 
 def test_get_topic_tree_args(sample_yaml_file):
